//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// August 13, 1999
//
// $Id: Phaser.h,v 1.4 2001/12/10 15:33:29 gvacanti Exp $
//
// Class to calculate a phase given an ephemeris. It operates on an
// event table that is suposed to contain:
//
//   double column TIME
//   attribute MJDREF
//   attribute TIMEZERO
//
// The phase is stored in the double column PHASE. The column is
// ovewritten if the DAL clobber mode is enabled.
//
// Based on an email by Julian Osborne (December 12, 1997), the
// absolute time of one event is
//
// t[n] = MJDREF + TIMEZERO + TIME[n] s
//
// Then
//
// PHASE[n] = refphase + (t[n] - refepoch) * f + 
//            0.5 (t[n] - refepoch)^2 * fdot 
//
#ifndef PHASER_H
#define PHASER_H

#include <STime.h>
#include <Dal.h>
#include <string>
#include <iostream>

class Phaser {

 public:
  Phaser(STime refepoch, double refphase = 0, double f = 1, double fdot = 0);
  ~Phaser();
  
  void setReferenceEpoch(STime refepoch);
  void setReferencePhase(double refphase);
  void setFrequency(double f);  	// Hz
  void setFrequencyDot(double fdot);	// Hz/s
  
  STime referenceEpoch() const;
  double referencePhase() const;
  double frequency() const;
  double frequencyDot() const;
  double frequencyTwoDot() const;
  double period() const;
  double periodDot() const;

  void phase(Table *) const;

  void print(ostream & os) const;
  
 private:
  STime _refepoch;
  double  _refphase, _f, _fdot, _f2dot;

  bool invariant() const;
  
};

inline ostream & operator<<(ostream & os, const Phaser & d)
{
  d.print(os);
  return os;
}

#endif
