
#include <param.h>
#include <Tracer.h>
#include <errstr.h>
#include <Dal.h>
#include <ExceptionErrorHandler.h>

#include "Phaser.h"

#include "phasecalc.h"

void phasecalc ()
{

  PROC
  double f = realParameter("frequency");
  double fdot = realParameter("frequencydot");
  STime epoch(stringParameter("epoch"));
  double phase = realParameter("phase");
  
  Phaser phaser(epoch, phase, f, fdot);
  errstr << "\n" << phaser << Sas::message(Msg::App, Msg::Sparse);
  vector<string> tables = stringListParameter("tables");

  ExceptionErrorHandler local;
  ErrorHandler* oldHandler = errHandler.handler(&local);
  errHandler.name(oldHandler->name());
  errHandler.verbosity(oldHandler->verbosity());

  for(unsigned i = 0; i < tables.size(); i++){
    DataSet * set = dataSetServer->open(tables[i], Dal::Modify);
    Table * tab = set->table(tables[i]);
    errstr << "Processing " << tables[i]
	   << Sas::message(Msg::App,Msg::Verbose);
    try{
      phaser.phase(tab);
    }
    catch (SasException::Error e) {
      errstr << "I detected an error while processing " + tab->qualifiedName()
	     << ". The error `" << e.code << "' said: " << e.message
	     << Sas::warning("SubProcessError");
      dataSetServer->close(set);
      continue;
    }
    dataSetServer->close(set);
  }
  errHandler.handler(oldHandler);
  ENDPROC
}
