//
// Giuseppe Vacanti, August 1999 -- European Space Agency
//
#include <string>
#include <vector>
#include <map>

#include <param.h>
#include <Tracer.h>
#include <errstr.h>
#include <Dal.h>
#include <ExprColumnGenerator.h>

#include "tabcalc.h"

void tabcalc ()
{
  //Bool = 1, Int8, Int16, Int32, Real32, Real64, DString

  PROC

  // These are the types for which it makes sense to work on
  std::map<std::string, Dal::DataType, less<std::string> > typemap;
  typemap["bool"]   = Dal::Bool;
  typemap["int8"]   = Dal::Int8;
  typemap["int16"]  = Dal::Int16;
  typemap["int32"]  = Dal::Int32;
  typemap["uint16"]  = Dal::Uint16;
  typemap["uint32"]  = Dal::Uint32;
  typemap["real32"] = Dal::Real32;
  typemap["real64"] = Dal::Real64;

  std::vector<std::string> tables = stringListParameter("tables");
  std::string
    expression = stringParameter("expression"),
    column = stringParameter("column"),
    unit = stringParameter("columnunit"),
    label = stringParameter("columnlabel"),
    dtype = stringParameter("columntype");

  // The parameter interface must make sure that the column type exists.
  assert(typemap.find(dtype) != typemap.end());

  ExprColumnGenerator cg(expression, column,
			 typemap[dtype]);

  for(unsigned i = 0; i < tables.size(); i++){
    DataSet * set = dataSetServer->open(tables[i], Dal::Modify);
    Table * tab = set->table(tables[i]);
    errstr << "Processing " << tables[i]
	   << ": " + column + " = " + expression
	   << Sas::message(Msg::App,Msg::Verbose);
    cg.doIt(*tab, *tab);

    Column * col = tab->column(column);
    col->units(unit);
    col->relabel(label);
    dataSetServer->close(set);  
  }
  ENDPROC
}
