//
//	phasecalctest.cc: test correctness of phasecalc task
//
//	U. Lammers and G. Vacancti
//

#include <cmath>
#include <cassert>
#include <string>
#include <vector>

#include <unistd.h>

#include "Tracer.h" // SAS
#include "param.h"
#include "errstr.h"
#include "Dal.h"
#include <STime.h>

#include "phasecalc.h"

using namespace std;

int main () 
{
  PROC
    
  std::string ds("phasecalc.ds");
  std::vector<std::string> pl;
  pl.push_back(ds + ":EVENTS");
  const unsigned int nrows = 100;
  const std::string e1("2000-01-01T00:00:00");
  const std::string e2("2000-01-01T00:00:01.333");
  STime tref(e1);

  DataSet * set = dataSetServer->open(ds, Dal::Create);
  Table * tab = set->addTable("EVENTS", nrows);
  Column * col = tab->addColumn("TIME", Dal::Real64);
  ColumnData * ptr = col->data();
  real64 * tcol = ptr->real64Data();  
  for(unsigned int i = 0; i < nrows; i++)
    tcol[i] = i;
  *tab->addAttribute("MJDREF") = tref.modifiedJulianDay().full();
  *tab->addAttribute("TIMEZERO") = 0;
  *tab->addAttribute("TIMESYS") = "UTC";
  delete ptr;
  dataSetServer->close(set); 

  openParameters("phasecalc");
  setParameter("tables", pl);
  setParameter("epoch", e1);
  setParameter("phase", 0.5);
  setParameter("frequency", 1.);
  setParameter("frequencydot", 2.);
  task(phasecalc);
  
  set = dataSetServer->open(ds, Dal::Read);
  tab = set->table("EVENTS");
  ptr = tab->column("PHASE")->data();
  real64 *pcol = ptr->real64Data();
  for(unsigned int i = 0; i < nrows; i++)
	assert(fabs(pcol[i] - 0.5) < 1e-6);
  delete ptr;
  dataSetServer->close(set);


  openParameters("phasecalc");
  setParameter("tables", pl);
  setParameter("epoch", e2);
  setParameter("phase", 0.);
  setParameter("frequency", 1.);
  setParameter("frequencydot", 2.);
  task(phasecalc);
  
  set = dataSetServer->open(ds, Dal::Read);
  tab = set->table("EVENTS");
  ptr = tab->column("PHASE")->data();
  ColumnData* ptrt = tab->column("TIME")->data();
  real64* ptcol = ptrt->real64Data();
  pcol = ptr->real64Data();
  for(unsigned int i = 0; i < nrows; i++){
	double iptr;
	double phase;
	double t = (ptcol[i] - 1.333)*(ptcol[i] - 0.333);
	phase = modf(t, &iptr);
	//	cout << t << " " << phase << " " << pcol[i] << endl;
	assert(fabs(pcol[i] - phase) < 1e-6);
  }
  delete ptr;
  dataSetServer->close(set);

  //unlink(ds.c_str());

  ENDPROC

}
