//
//	tabcalctest.cc: test correctness of tabcalc task
//
//	U. Lammers and G. Vacancti
//

#include <cmath>
#include <cassert>
#include <string>
#include <vector>

#include <unistd.h>

#include "Tracer.h" // SAS
#include "param.h"
#include "errstr.h"
#include "Dal.h"

#include <tabcalc.h>

int main () 
{
  PROC
    
  dataSetServer->client("tabcalc.test");
  const unsigned int nrows = 100;
  
  DataSet *set = dataSetServer->open("tabcalc.ds", Dal::Create);
  Table *tab = set->addTable("TABCALC", nrows);
  Column *col = tab->addColumn("COL", Dal::Int32);
  ColumnData *ptr = col->data();
  
  int32 *icol = ptr->int32Data();
  
  for(unsigned int i = 0; i < nrows; i++)
    icol[i] = i;

  delete ptr;
  dataSetServer->close(set);

  // RESCOL = COL * 2
  std::vector<std::string> pl;
  pl.push_back("tabcalc.ds:TABCALC");
  openParameters("tabcalc");
  setParameter("tables",pl);
  setParameter("column","RESCOL");
  setParameter("columntype", "real64");
  setParameter("expression","COL*2");
  setParameter("columnunit","unit");
  setParameter("columnlabel", "label");
  task(tabcalc);

  set = dataSetServer->open("tabcalc.ds", Dal::Read);
  tab = set->table("TABCALC");
  col = tab->column("RESCOL");
  ptr = col->data();
  real64 * dcol = ptr->real64Data();
  for(unsigned int i = 0; i < nrows; i++)
    assert(dcol[i] == 2.*i);

  assert(col->units() == "unit");
  assert(col->label() == "label");
  
  dataSetServer->close(set);  

  // same thing, integer result, also clobber
  openParameters("tabcalc");
  setParameter("tables",pl);
  setParameter("column","RESCOL");
  setParameter("columntype", "int32");
  setParameter("expression","COL*2");
  task(tabcalc);

  set = dataSetServer->open("tabcalc.ds", Dal::Read);
  tab = set->table("TABCALC");
  ptr = tab->column("RESCOL")->data();
  int32 * jcol = ptr->int32Data();
  for(unsigned int i = 0; i < nrows; i++)
    assert(jcol[i] == 2.*i);
  
  dataSetServer->close(set);  

  //  std::unlink("tabcalc.ds");

  ENDPROC
}
